use 5.010;
use strict;
use warnings;
package Benchmark::Perl::Formance::Analyzer;
# git description: v0.001-2-g0338102

BEGIN {
  $Benchmark::Perl::Formance::Analyzer::AUTHORITY = 'cpan:SCHWIGON';
}
# ABSTRACT: turns baubles into trinkets
$Benchmark::Perl::Formance::Analyzer::VERSION = '0.002';
1;

__END__

=pod

=encoding utf-8

=head1 NAME

Benchmark::Perl::Formance::Analyzer - turns baubles into trinkets

=head1 AUTHOR

Steffen Schwigon <ss5@renormalist.net>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2015 by Steffen Schwigon.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
